/* (C) 2009 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate IP address details.
 *
 * $Id: AddrInfo.cc,v 1.7 2013/02/09 11:03:11 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <string.h>
#include "AddrInfo.hh"


namespace HTTPTunnel {


// Construction/destruction
AddrInfo::AddrInfo(std::string host)
: m_host(host), m_family(UNKNOWN_FAMILY), m_addrLen(0), m_addr(), m_addrStr("")
{
}

AddrInfo::AddrInfo(const AddrInfo& that)
: m_host(that.m_host), m_family(that.m_family), m_addrLen(that.m_addrLen),
  m_addr(that.m_addr), m_addrStr(that.m_addrStr)
{
}

AddrInfo::~AddrInfo()
{
}


// Assignment
AddrInfo& AddrInfo::operator=(const AddrInfo& that)
{
	// Self assignment
	if(this != &that) {
		m_host = that.m_host;
		m_family = that.m_family;
		m_addrLen = that.m_addrLen;
		m_addr = that.m_addr;
		m_addrStr = that.m_addrStr;
	}

	return *this;
}


/*************/
/* Accessors */
/*************/


// Get hostname as a string
std::string AddrInfo::getHostName(void) const
{
	return m_host;
}

// Get address family
AddrInfo::Family AddrInfo::getFamily(void) const
{
	return m_family;
}

// Get address family as a string
std::string AddrInfo::getFamilyStr(void) const
{
	return getFamilyStr(m_family);
}

// Get sockaddr len
size_t AddrInfo::getAddrLen(void) const
{
	return m_addrLen;
}

// Get pointer to generic sockaddr
const struct sockaddr_storage* AddrInfo::getAddr(void) const
{
	return &m_addr;
}

// Get IP address as a string
std::string AddrInfo::getAddrStr(void) const
{
	return m_addrStr;
}


/************/
/* Mutators */
/************/

// Set address family
void AddrInfo::setFamily(int family)
{
	if(family == AF_INET)
		m_family = IPV4;

	#if _POSIX_C_SOURCE >= 1 || _XOPEN_SOURCE || _POSIX_SOURCE
	else if(family == AF_INET6)
		m_family = IPV6;
	#endif

	else
		m_family = UNKNOWN_FAMILY;
}

// Set sockaddr details
void AddrInfo::setAddr(size_t addrLen, const struct sockaddr* addr)
{
	m_addrLen = addrLen;
	memcpy(&m_addr, addr, addrLen);
}

// Set IP address as a string
void AddrInfo::setAddrStr(std::string addrStr)
{
	m_addrStr = addrStr;
}


/***********/
/* Utility */
/***********/

// Get address family constant as a string
std::string AddrInfo::getFamilyStr(const Family& family)
{
	switch(family) {
		case(IPV4):
			return "IPV4";
			break;
		case(IPV6):
			return "IPV6";
			break;
		default:
			break;
	}

	return "UNKNOWN_FAMILY";
}


/******************/
/* Helper Methods */
/******************/


// Output to a stream
std::ostream& operator<<(std::ostream& str, const AddrInfo& addrInfo)
{
	str << "Hostname: " << addrInfo.getHostName() << std::endl;
	str << "Family: " << addrInfo.getFamilyStr() << std::endl;
	str << "Addr: " << addrInfo.getAddr() << ", AddrLen: " << addrInfo.getAddrLen() << std::endl;
	str << "AddrStr: " << addrInfo.getAddrStr() << std::endl;

	return str;
}


}	// namespace

