/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * A convenient place to store all global configuration for
 * a client tunneler.
 *
 * $Id: ClientSideConfig.cc,v 1.4 2012/12/27 13:26:45 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <sstream>
#include "ClientSideConfig.hh"
#include "ProxySettings.hh"
#include "Utils.hh"
#include "ClientAuthenticator.hh"
#include "ServerAuthSettings.hh"


namespace HTTPTunnel {


// Construction/destruction
ClientSideConfig::ClientSideConfig(Enums::Protocol prot, bool ssl, bool sockTun,
	IPAddr listenIP, IPAddr serverIP, std::string suser, std::string spasswd,
	std::string dhost, unsigned int dport, std::string tune)
: SideConfig(prot, ssl, sockTun, listenIP), m_serverIP(serverIP), m_resolvedServerIP(),
  m_fixedDestIP(dhost, dport), m_proxySettings(0), m_tune(tune)
{
	/* Direct to server, fixed tunnel */

	// Try and resolve server IP address
	m_resolvedServerIP = Utils::resolveIP(m_serverIP.getHost());

	// Create authenticator
	m_authenticator = new ClientAuthenticator();
	if(suser != "" || spasswd != "")
		setClientAuth(suser, spasswd);
}

ClientSideConfig::ClientSideConfig(Enums::Protocol prot, bool ssl, bool sockTun,
	IPAddr listenIP, IPAddr serverIP, std::string suser, std::string spasswd,
	std::string dhost, unsigned int dport, std::string tune, IPAddr proxyIP,
	std::string puser, std::string ppasswd)
: SideConfig(prot, ssl, sockTun, listenIP), m_serverIP(serverIP), m_resolvedServerIP(),
  m_fixedDestIP(dhost, dport), m_proxySettings(0), m_tune(tune)
{
	/* Via explict (possibly authenticating) proxy, fixed tunnel */

	// Create proxy authentication settings
	std::string pdomain = lookupUserDomain(puser);
	m_proxySettings = new ProxySettings(puser, pdomain, ppasswd, proxyIP);

	// Don't bother resolving server IP address
	m_resolvedServerIP = m_serverIP.getHost();

	// Create authenticator
	m_authenticator = new ClientAuthenticator();
	if(suser != "" || spasswd != "")
		setClientAuth(suser, spasswd);
}

ClientSideConfig::ClientSideConfig(Enums::Protocol prot, bool ssl, IPAddr listenIP,
	IPAddr serverIP, std::string suser, std::string spasswd, std::string tune)
: SideConfig(prot, ssl, false, listenIP), m_serverIP(serverIP), m_resolvedServerIP(),
  m_fixedDestIP(), m_proxySettings(0), m_tune(tune)
{
	/* Direct to server, SOCKS tunnel */

	// Try and resolve server IP address
	m_resolvedServerIP = Utils::resolveIP(m_serverIP.getHost());

	// Create authenticator
	m_authenticator = new ClientAuthenticator();
	if(suser != "" || spasswd != "")
		setClientAuth(suser, spasswd);
}

ClientSideConfig::ClientSideConfig(Enums::Protocol prot, bool ssl, IPAddr listenIP,
	IPAddr serverIP, std::string suser, std::string spasswd, std::string tune,
	IPAddr proxyIP, std::string puser, std::string ppasswd)
: SideConfig(prot, ssl, false, listenIP), m_serverIP(serverIP), m_resolvedServerIP(),
  m_fixedDestIP(), m_proxySettings(0), m_tune(tune)
{
	/* Via explict (possibly authenticating) proxy, SOCKS tunnel */

	// Create proxy authentication settings
	std::string pdomain = lookupUserDomain(puser);
	m_proxySettings = new ProxySettings(puser, pdomain, ppasswd, proxyIP);

	// Don't bother resolving server IP address
	m_resolvedServerIP = m_serverIP.getHost();

	// Create authenticator
	m_authenticator = new ClientAuthenticator();
	if(suser != "" || spasswd != "")
		setClientAuth(suser, spasswd);
}

ClientSideConfig::~ClientSideConfig()
{
	// Free up proxy/serverauth settings
	if(m_proxySettings) {
		delete m_proxySettings;
		m_proxySettings = 0;
	}
}


/*************/
/* Accessors */
/*************/


// Client config?
bool ClientSideConfig::isClient(void) const
{
	return true;
}

// Are we going via a proxy?
bool ClientSideConfig::viaProxy(void) const
{
	return (m_proxySettings != 0);
}

// Are we using a fixed tunnel?
bool ClientSideConfig::fixed(void) const
{
	return (!m_fixedDestIP.getHost().empty());
}

// Are we running a SOCKS proxy?
bool ClientSideConfig::socks(void) const
{
	return !fixed();
}

// Get address for server end of tunnel
IPAddr ClientSideConfig::getServerIP(void) const
{
	return m_serverIP;
}

// Get resolved address for server end of tunnel
std::string ClientSideConfig::getResolvedServerIP(void) const
{
	return m_resolvedServerIP;
}

// Get address for fixed tunnel destination
IPAddr ClientSideConfig::getFixedDestIP(void) const
{
	return m_fixedDestIP;
}

// Get address for next hop in tunnel
IPAddr ClientSideConfig::getNextHopIP(void) const
{
	if(viaProxy())
		return getProxySettings()->getIP();
	else
		return getServerIP();
}

// Proxy settings
const ProxySettings* ClientSideConfig::getProxySettings(void) const
{
	return m_proxySettings;
}

// Tunnel tuning
std::string ClientSideConfig::getTunTune(void) const
{
	return m_tune;
}


/************/
/* Mutators */
/************/


// Set client authorisation
void ClientSideConfig::setClientAuth(std::string user, std::string passwd)
{
	// Save username & passwd
	ClientAuthenticator* auth = dynamic_cast<ClientAuthenticator*>(m_authenticator);
	auth->setUserPasswd(user, passwd);
}


/********************/
/* Public Interface */
/********************/


// Serialize to a file
void ClientSideConfig::serialize(std::ofstream& file)
{
	// Client header
	file << "#######################" << std::endl;
	file << "## Client configuration" << std::endl;
	file << "#######################" << std::endl;
	file << std::endl;
	file << "[CLIENT]" << std::endl;

	// What sort of client?
	if(fixed()) {
		// Fixed tunnel
		file << "fixed " << m_fixedDestIP.getHost() << " " << m_fixedDestIP.getPort();
		if(getProtocol() == Enums::UDP)
			file << " udp";
		file << std::endl;
	}
	else {
		// SOCKS tunnel
		file << "socks" << std::endl;
	}

	// Local listening address
	file << "listen ";
	std::string host = m_listenIP.getHost();
	if(!host.empty())
		file << host << " ";
	file << m_listenIP.getPort();
	if(host.empty())
		file << " " << m_listenIP.getFamilyStr();
	file << std::endl;

	// Tunnel server location
	file << "tunsvr " << m_serverIP.getHost() << " " << m_serverIP.getPort() << std::endl;

	// Proxy settings
	if(viaProxy()) {
		const ProxySettings* proxyCfg = getProxySettings();
		IPAddr proxyip = proxyCfg->getIP();
		file << "proxy " << proxyip.getHost() << " " << proxyip.getPort();
		file << std::endl;
		if(proxyCfg->gotUserPasswd()) {
			std::stringstream user;
			if(proxyCfg->gotDomain())
				user << proxyCfg->getDomain() << "\\";
			user << proxyCfg->getUser();
			file << "proxyauth " << user.str() << " " << proxyCfg->getPasswd();
			file << std::endl;
		}
	}

	// Client authentication
	ServerAuthSettings* svrAuth = m_authenticator->getDefaultAuthSettings();
	if(svrAuth->gotUserPasswd()) {
		file << "cliauth " << svrAuth->getUser() << " " << svrAuth->getPasswd();
		file << std::endl;
	}

	// SSL certificate
	if(m_ssl)
		file << "ssl" << std::endl;

	// Tuning
	if(m_tune != "")
		file << "tune " << m_tune << std::endl;

	file << std::endl;
}


/***********************/
/* Protected Interface */
/***********************/


// Get domain for user
std::string ClientSideConfig::lookupUserDomain(std::string& user)
{
	// If username contains a '\', it is a combined domain & username
	std::string domain = "";
	std::vector<std::string> split;
	Utils::SplitString(user, "\\", split);
	if(split.size() == 2) {
		domain = split[0];
		user = split[1];
	}

	return domain;
}


}	// namespace

