/* (C) 2011 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate an IP hostess (actually, a TCP/UDP hostess).
 *
 * $Id: IPAddr.cc,v 1.1 2012/12/27 13:53:02 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <iostream>
#include "IPAddr.hh"
#include "Utils.hh"

namespace HTTPTunnel {

// Construction/destruction
IPAddr::IPAddr()
: m_host(""), m_port(0), m_family(AddrInfo::UNKNOWN_FAMILY)
{
}

IPAddr::IPAddr(std::string host, unsigned int port)
: m_host(host), m_port(port), m_family(AddrInfo::UNKNOWN_FAMILY)
{
	// Work out address family
	setFamilyFromHost(AddrInfo::IPV4);
}

IPAddr::IPAddr(std::string host, unsigned int port, AddrInfo::Family family)
: m_host(host), m_port(port), m_family(AddrInfo::UNKNOWN_FAMILY)
{
	// Work out address family
	setFamilyFromHost(family);
}

IPAddr::IPAddr(const IPAddr& that)
: m_host(that.m_host), m_port(that.m_port), m_family(that.m_family)
{
}

IPAddr::~IPAddr()
{
}

// Assignment
IPAddr& IPAddr::operator=(const IPAddr& that)
{
	if(this != &that) {
		m_host = that.m_host;
		m_port = that.m_port;
		m_family = that.m_family;
	}

	return *this;
}

// Equality
bool IPAddr::operator==(const IPAddr& that) const
{
	// All fields must match
	bool res = (m_host == that.m_host);
	res &= (m_port == that.m_port);
	res &= (m_family == that.m_family);

	return res;
}


/*************/
/* Accessors */
/*************/

// Get host
const std::string& IPAddr::getHost(void) const
{
	return m_host;
}

// Get port
unsigned int IPAddr::getPort(void) const
{
	return m_port;
}

// Get address family
AddrInfo::Family IPAddr::getFamily(void) const
{
	return m_family;
}

// Get address family as a string
std::string IPAddr::getFamilyStr(void) const
{
	return AddrInfo::getFamilyStr(m_family);
}


/************/
/* Mutators */
/************/

// Set port
void IPAddr::setPort(unsigned int port)
{
	m_port = port;
}


/*********************/
/* Private Interface */
/*********************/


// Set address family from host
void IPAddr::setFamilyFromHost(AddrInfo::Family defaultFamily)
{
	// Set default for when we can't work it out
	m_family = defaultFamily;

	// Try and work it out from resolved address info
	if(!m_host.empty() && m_host != "-") {
		try {
			AddrInfo addrInfo = Utils::getAddrInfo(m_host);
			m_family = addrInfo.getFamily();
		}
		catch(AddrLookupError&) {}
	}
}


/******************/
/* Helper Methods */
/******************/


// Output to a stream
std::ostream& operator<<(std::ostream& str, const IPAddr& addr)
{
	if(addr.getHost().empty())
		str << "<No Host>";
	else
		str << addr.getHost();

	str << ";" << addr.getPort() << " ";

	if(addr.getFamily() == AddrInfo::IPV4)
		str << "(IPV4)";
	else if(addr.getFamily() == AddrInfo::IPV6)
		str << "(IPV6)";
	else
		str << "(Unknown)";

	return str;
}


}	// namespace


