/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate a UDP socket.
 *
 * $Id: UDPSock.cc,v 1.10 2012/12/27 13:25:29 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <arpa/inet.h>
#include <netdb.h>
#include <cerrno>
#include <cstring>
#include <strings.h>
#include "UDPSock.hh"
#include "Logger.hh"
#include "Debug.hh"
#include "Tunneler.hh"
#include "Utils.hh"


namespace HTTPTunnel {


// Global references
extern Logger* g_pLogger;


// Construction/destruction
UDPSock::UDPSock(bool blocking) throw(SockError)
: Sock(blocking), m_sockAddr(), m_sockAddrLen(0), m_ownSocket(true)
{
}

UDPSock::UDPSock(int fd, socklen_t sockAddrLen, std::string cliAddr) throw(SockError)
: Sock(fd, cliAddr), m_sockAddr(), m_sockAddrLen(sockAddrLen),
  m_ownSocket(false)
{
}

UDPSock::~UDPSock()
{
	DPRINTV("UDPSock::~UDPSock()");

	MutexVar mv(m_mtx);

	// Should we close?
	if(m_ownSocket) {
		try {
			// Close socket descriptor
			close();
		}
		catch(SockError& e) {
			g_pLogger->info(e.what());
		}
	}
}


/********************/
/* Public Interface */
/********************/


// Wait for a new incoming UDP connection
void UDPSock::newConnection(Tunneler* tunneler) throw(SockError)
{
	MutexVar mv(m_mtx);

	// Is this really a new UDP connection, or simply a continuation?
	if(tunneler->getNoActiveTunnels() == 0) {
		// Accept connection
		Sock* connectedSock = accept();

		try {
			// Tell tunneler
			if(tunneler)
				tunneler->newUDPConnectionArrived(connectedSock);
			else
				throw SockError("No tunneler to handle new UDP connection", 0);
		}
		catch(TunnelError& e) {
			g_pLogger->info(e.what());
			delete connectedSock;
		}
	}
}


// Set socket listening
void UDPSock::listen(void) throw(SockError)
{
	// Do nothing for UDP
}


// Wait for a new incoming connection
Sock* UDPSock::accept(void) throw(SockError)
{
	// Return connected UDP socket
	return new UDPSock(m_fd, m_sockAddrLen, "Unknown");
}


// Establish connection out
void UDPSock::connect(IPAddr& ipaddr) throw(SockError, CommsError)
{
	// Create underlying socket
	createSockFD(ipaddr, SOCK_DGRAM);

	// Get address details
	std::string addr = ipaddr.getHost();
	unsigned int port = ipaddr.getPort();

	MutexVar mv(m_mtx);

	// Lookup destination address
	try {
		AddrInfo addrInfo = Utils::getAddrInfo(addr);
		m_sockAddrLen = addrInfo.getAddrLen();

		// Set up server address structure
		memcpy(&m_sockAddr, addrInfo.getAddr(), m_sockAddrLen);
		if(addrInfo.getFamily() == AddrInfo::IPV6)
			((struct sockaddr_in6*)&m_sockAddr)->sin6_port = htons(port);
		else
			((struct sockaddr_in*)&m_sockAddr)->sin_port = htons(port);
	}
	catch(AddrLookupError& e) {
		std::stringstream err;
		err << "Failed to resolve UDP server address " << addr << ": " << e.what();
		throw ConnectError(err.str());
	}
}


// Read from socket
int UDPSock::read(unsigned char* buff, size_t buffSize)
	throw(SockError, CommsError)
{
	MutexVar mv(m_mtx);

	// Read data from socket
again:
	ssize_t noread = recvfrom(m_fd, buff, buffSize, 0, (struct sockaddr*)&m_sockAddr, &m_sockAddrLen);
	if(noread < 0) {
		if(errno == EINTR)
			goto again;
		else
			throw SockError("Read error from UDP socket", errno);
	}

	// Get connected client address
	m_connectedClientAddr = Utils::getNameInfo((struct sockaddr*)&m_sockAddr, m_sockAddrLen);

	return noread;
}


// Write to socket
int UDPSock::write(const unsigned char* buff, size_t buffSize)
	throw(SockError, CommsError)
{
	MutexVar mv(m_mtx);

	// Write data to socket
again:
	ssize_t nowritten = sendto(m_fd, buff, buffSize, 0, (struct sockaddr*)&m_sockAddr, m_sockAddrLen);
	if(nowritten < 0) {
		if(errno == EINTR)
			goto again;
		else
			throw SockError("Write error to UDP socket", errno);
	}

	return nowritten;
}


// Get the result of a non-blocking connect attempt
void UDPSock::getConnectResult(bool& resultAvailable, bool& connected)
	throw(SockError)
{
	// Do nothing for UDP
	resultAvailable = connected = true;
}


// Bind socket to an address
void UDPSock::bind(IPAddr& ipaddr) throw(SockError)
{
	// Create underlying socket
	createSockFD(ipaddr, SOCK_DGRAM);

	// Do the bind
	Sock::bind(ipaddr);

	// Set socket address size appropriately
	MutexVar mv(m_mtx);
	m_sockAddrLen = m_listenAddrLen;
}


}	// namespace

